# libwtrOnlineDemoForAndroid
This is a demo app to control the WTROnline device which run [libwtrOnline]()

## How to Import

- And add a dependency code to your app module's `build.gradle` file.

  ```
  dependencies {
    implementation fileTree(dir: 'libs', include: '*.aar') // 如果有aar，则添加AAR依赖,把aar文件存放在目录app/libs
  }
   ```
## Usage

### 初始化蓝牙

```
    BluetoothDevice mDevice = ...;//扫描获取到蓝牙
    BluetoothGattCallback gattCallback = new InnerGattCallback();//监听蓝牙状态回调
    //连接蓝牙.
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
        mDevice.connectGatt(getApplicationContext(), false, new InnerGattCallback(), BluetoothDevice.TRANSPORT_LE);
    } else {
        mDevice.connectGatt(getApplicationContext(), false, new InnerGattCallback());
    }
    
```

### 监听蓝牙状态

处理蓝牙BluetoothGattCallback的回调, 初始化设备。

```
public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
  if (status == BluetoothGatt.GATT_SUCCESS) {
        if (newState == BluetoothProfile.STATE_CONNECTED) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                gatt.requestConnectionPriority(BluetoothGatt.CONNECTION_PRIORITY_HIGH);
            }
            gatt.discoverServices();//初始化蓝牙的Services
        }
  }
  if (status == BluetoothGatt.GATT_SUCCESS) {
      switch (newState) {
          case BluetoothProfile.STATE_CONNECTED:
              break;
          case BluetoothProfile.STATE_DISCONNECTED:
              gatt.close();
              if (mClinet != null) {
                  mClinet.close();
              }
              break;
      }
  } else {
      gatt.close();
      if (mClinet != null) {
          mClinet.close();
      }
  }
}
public void onServicesDiscovered(BluetoothGatt gatt, int status) {
    try {
        mClinet = new WTROnlineClient(getApplicationContext(), gatt, status);
        mCallback = new WTROnlineCallbackMain();
        mClinet.setBleCallback(mCallback);
        mCallback.onGattPrepared(mClinet, gatt);//, service, writeChar, notifyChar);
    }catch (RuntimeException e){
        gatt.disconnect();
        updateMessage(String.format(Locale.ENGLISH, e.getMessage() + "; status = %d", status), false);
    }
}

```

### 监听设备状态回调

```
private class WTROnlineCallbackMain extends WTROnlineCallback {

        @Override
        public void onGattPrepared(
                WTROnlineClient client,
                BluetoothGatt gatt
        ) {
            updateMessage("Discover service and characteristics success", false);
            int mtu = WTConstants.DEFAULT_MTU_LENGTH;
            mLog.d("Request MTU " + mtu);
            //必须设置mtu,默认值不足以发送最长数据,可根据实际应用调整
            boolean requestMtu = gatt.requestMtu(mtu);
            if (!requestMtu) {
                mLog.w("Request mtu failed");
                updateMessage(String.format(Locale.ENGLISH, "Request mtu %d failed", mtu), false);
                onGattServiceCharacteristicDiscovered();
            }
            super.onGattPrepared(client, gatt);
        }


        @Override
        public void onError(WTROnlineClient client, int errCode) {
        }
        
        //接收蓝牙配置wifi返回
        @Override
        public void onReceiveConfigNetworkResult(com.waytronic.libwtronline.WTROnlineClient client, boolean success) {
            super.onReceiveConfigNetworkResult(client, success);
            final String message = success ? "配置发送成功,请等待指示灯变化" : "配置失败";
            // 通知 WiFiConfigDialogFragment 配置结果
            ((WiFiConfigDialogFragment)getSupportFragmentManager().findFragmentByTag("WiFiConfigDialog"))
                    .notifyConfigResult(success, message);
            updateMessage(message, false);
            disconnectGatt();
        }
}
```

### 调用设备方法示例

```
  mClient.startConfigNetwork(wifiName, wifiPassword);//配置wifi名称和密码
  mClinet.requestCloseConnection();//主动断开连接
```