package com.waytronic.wtronline_demo.app;

import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.os.Handler;
import android.os.Looper;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;

public class WTPcmPlayer {
    private AudioTrack audioTrack;
    public boolean ismIsPlaying(){
        return  mIsPlaying;
    }
    private boolean mIsPlaying = false;
    private final Handler handler = new Handler(Looper.getMainLooper());

    public void startPlayFile(String filePath) {
        // 配置AudioTrack参数
        int channelConfig = AudioFormat.CHANNEL_OUT_MONO;
        int audioFormat = AudioFormat.ENCODING_PCM_16BIT;
        int sampleRate = 16000; // 8kHz采样率

        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            try {
                byte[] bytes = Files.readAllBytes(Paths.get(filePath));
                audioTrack = new AudioTrack(AudioManager.STREAM_MUSIC, sampleRate,
                        channelConfig, audioFormat, bytes.length, AudioTrack.MODE_STATIC);

                handler.post(() -> {
                    if (audioTrack != null && mIsPlaying) {
                        audioTrack.write(bytes, 0, bytes.length);
                        // 开始播放
                        audioTrack.play();
//                        audioTrack.stop();
//                        mIsPlaying = false;
                    }
                });
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        mIsPlaying = true;
    }
    public void startPlayback(int bufferSize) {
        // 配置AudioTrack参数
        int channelConfig = AudioFormat.CHANNEL_OUT_MONO;
        int audioFormat = AudioFormat.ENCODING_PCM_16BIT;
        int sampleRate = 16000; // 8kHz采样率

        // 计算最小缓冲区大小
        int minBufferSize = AudioTrack.getMinBufferSize(sampleRate, channelConfig, audioFormat);
        if (bufferSize < minBufferSize) {
            throw new IllegalArgumentException("Buffer size is too small for the given parameters.");
        }

        audioTrack = new AudioTrack(AudioManager.STREAM_MUSIC, sampleRate,
                channelConfig, audioFormat, bufferSize, AudioTrack.MODE_STREAM);

        // 开始播放
        audioTrack.play();
        mIsPlaying = true;
    }

    public void writeData(byte[] pcmData) {
        // 在这里，你应该替换为实际从蓝牙接收PCM数据的逻辑
        // 假设pcmData是每次接收的数据，你需要将其加入到播放队列
        // 在主线程更新UI或处理逻辑
        handler.post(() -> {
            if (audioTrack != null && mIsPlaying) {
                audioTrack.write(pcmData, 0, pcmData.length);
            }
        });

        // 根据实际情况决定是否继续接收下一批数据
        // 例如，使用循环或定时器，确保不阻塞主线程
    }

    public void stopPlayback() {
        if (audioTrack != null) {
            mIsPlaying = false;
            audioTrack.stop();
            audioTrack.release();
            audioTrack = null;
        }
    }
}
