package com.waytronic.wtronline_demo.ui;

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothProfile;
import android.content.Intent;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import com.waytronic.libwtronline.WTROnlineCallback;
import com.waytronic.libwtronline.WTROnlineClient;
import com.waytronic.libwtronline.WTROnlineParameter;

import com.waytronic.wtdemo.R;
import com.waytronic.wtdemo.databinding.WtActivityBinding;
import com.waytronic.wtdemo.databinding.WtContentBinding;
import com.waytronic.wtdemo.databinding.WtMessageItemBinding;
import com.waytronic.wtronline_demo.app.BaseActivity;
import com.waytronic.wtronline_demo.app.WTLog;
import com.waytronic.wtronline_demo.constants.WTConstants;


import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

@SuppressLint("MissingPermission")
public class WTActivity extends BaseActivity implements WiFiConfigListener {
    private static final int REQUEST_CONFIGURE = 0x20;
    private static final String TAG = "WTActivity";
    private final WTLog mLog = new WTLog(getClass());

    private BluetoothDevice mDevice;
    private WTROnlineClient mClient;
    private volatile boolean mConnected;

    private List<Message> mMsgList;
    private MsgAdapter mMsgAdapter;

    private WtContentBinding mContent;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        WtActivityBinding mBinding = WtActivityBinding.inflate(getLayoutInflater());
        setContentView(mBinding.getRoot());
        setSupportActionBar(mBinding.toolbar);
        setHomeAsUpEnable(true);

        mDevice = getIntent().getParcelableExtra(WTConstants.KEY_BLE_DEVICE);
        assert mDevice != null;
        String deviceName = mDevice.getName() == null ? getString(R.string.string_unknown) : mDevice.getName();
        setTitle(deviceName);

        mContent = mBinding.content;

        mMsgList = new LinkedList<>();
        mMsgAdapter = new MsgAdapter();
        mContent.recyclerView.setAdapter(mMsgAdapter);

        WTButtonListener clickListener = new WTButtonListener();

        mContent.blufiConnect.setOnClickListener(clickListener);
        mContent.blufiConnect.setOnLongClickListener(clickListener);

        mContent.blufiDisconnect.setOnClickListener(clickListener);
        mContent.blufiDisconnect.setOnLongClickListener(clickListener);
        mContent.blufiDisconnect.setEnabled(false);

        mContent.blufiStartWificonfig.setOnClickListener(clickListener);
        mContent.blufiStartWificonfig.setOnLongClickListener(clickListener);
        mContent.blufiStartWificonfig.setEnabled(false);

    }

    @Override
    protected void onDestroy() {
        super.onDestroy();

        if (mClient != null) {
            mClient.requestCloseConnection();
            mClient = null;
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == REQUEST_CONFIGURE) {
            if (!mConnected) {
                return;
            }
            if (resultCode == RESULT_OK) {

            }

            return;
        }

        super.onActivityResult(requestCode, resultCode, data);
    }

    private void updateMessage(String message, boolean isNotificaiton) {
        runOnUiThread(() -> {
            Message msg = new Message();
            msg.text = message;
            msg.isNotification = isNotificaiton;
            mMsgList.add(msg);
            mMsgAdapter.notifyItemInserted(mMsgList.size() - 1);
            mContent.recyclerView.scrollToPosition(mMsgList.size() - 1);
        });

    }

    /**
     * Try to connect device
     */
    private void connect() {
        mContent.blufiConnect.setEnabled(false);

        if (mClient != null) {
            mClient.requestCloseConnection();
            mClient = null;
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            mDevice.connectGatt(getApplicationContext(), false, new InnerGattCallback(), BluetoothDevice.TRANSPORT_LE);
        } else {
            mDevice.connectGatt(getApplicationContext(), false, new InnerGattCallback());
        }
    }

    /**
     * Request device disconnect the connection.
     */
    private void disconnectGatt() {
        mContent.blufiDisconnect.setEnabled(false);

        if (mClient != null) {
            mClient.requestCloseConnection();
        }
    }
    @Override
    public void startWiFiConfiguration(String wifiName, String wifiPassword) {
        if (mClient == null){
            return;
        }
        mClient.startConfigNetwork(wifiName, wifiPassword);
    }
    private void onGattConnected() {
        mConnected = true;
        runOnUiThread(() -> {
            mContent.blufiConnect.setEnabled(false);
            mContent.blufiDisconnect.setEnabled(true);
        });
    }

    private void onGattServiceCharacteristicDiscovered() {
        runOnUiThread(() -> {
            mContent.blufiStartWificonfig.setEnabled(true);
        });
    }

    private void onGattDisconnected() {
        mConnected = false;
        runOnUiThread(() -> {
            mContent.blufiConnect.setEnabled(true);
            mContent.blufiDisconnect.setEnabled(false);
            mContent.blufiStartWificonfig.setEnabled(false);
        });
    }

    //蓝牙回调
    private class InnerGattCallback extends BluetoothGattCallback {
        WTROnlineCallback mCallback;
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                if (newState == BluetoothProfile.STATE_CONNECTED) {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                        gatt.requestConnectionPriority(BluetoothGatt.CONNECTION_PRIORITY_HIGH);
                    }
                    gatt.discoverServices();
                }
            }

            String devAddr = gatt.getDevice().getAddress();
            mLog.d(String.format(Locale.ENGLISH, "onConnectionStateChange addr=%s, status=%d, newState=%d",
                    devAddr, status, newState));
            if (status == BluetoothGatt.GATT_SUCCESS) {
                switch (newState) {
                    case BluetoothProfile.STATE_CONNECTED:
                        onGattConnected();
                        updateMessage(String.format("Connected %s", devAddr), false);
                        break;
                    case BluetoothProfile.STATE_DISCONNECTED:
                        gatt.close();
                        onGattDisconnected();
                        if (mClient != null) {
                            mClient.close();
                            mClient = null;
                        }
                        updateMessage(String.format("Disconnected %s", devAddr), false);
                        break;
                }
            } else {
                gatt.close();
                onGattDisconnected();
                if (mClient != null) {
                    mClient.close();
                    mClient = null;
                }
                updateMessage(String.format(Locale.ENGLISH, "Disconnect %s, status=%d", devAddr, status),
                        false);
            }
        }

        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            mLog.d(String.format(Locale.ENGLISH, "onServicesDiscovered status=%d", status));
            try {
                mClient = new WTROnlineClient(getApplicationContext(), gatt, status);
                mCallback = new WTROnlineCallbackMain();
                mClient.setBleCallback(mCallback);
                mCallback.onGattPrepared(mClient, gatt);//, service, writeChar, notifyChar);
            }catch (RuntimeException e){
                gatt.disconnect();
                updateMessage(String.format(Locale.ENGLISH, e.getMessage() + "; status = %d", status), false);
            }
        }
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            //接收到原始数据
//            mLog.d(String.format(Locale.ENGLISH, "onCharacteristicChanged data:%s",bytesToHex(characteristic.getValue())));
            if (mClient == null){
                mLog.d("客户端未初始化");
                return;
            }
            if (characteristic.getUuid().compareTo(WTROnlineParameter.UUID_NETWORK_NOTIFY_CHARACTERISTIC) == 0) {
                byte[] data = characteristic.getValue();
                mLog.d("Gatt Notification: " + bytesToHex(data));
                try {
                    updateMessage(String.format(Locale.ENGLISH, "收到数据 %s", bytesToHex(characteristic.getValue())), false);
                    mClient.decodeData(data);
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }

            }

        }

        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            if (status != BluetoothGatt.GATT_SUCCESS) {
                gatt.disconnect();
                updateMessage(String.format(Locale.ENGLISH, "WriteChar error status %d", status), false);
            }else {
                updateMessage(String.format(Locale.ENGLISH, "%s 发出数据 %s", characteristic.getUuid().toString(), bytesToHex(characteristic.getValue())), false);
            }
        }

        @TargetApi(Build.VERSION_CODES.LOLLIPOP)
        public void onMtuChanged(BluetoothGatt gatt, int mtu, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
//                mBlufiMTU = mtu - 4; // Three bytes BLE header, one byte reserved
            }

            mLog.d(String.format(Locale.ENGLISH, "onMtuChanged status=%d, mtu=%d", status, mtu));
            if (status == BluetoothGatt.GATT_SUCCESS) {
                updateMessage(String.format(Locale.ENGLISH, "Set mtu complete, mtu=%d ", mtu), false);
            } else {
                mClient.setPostPackageLengthLimit(20);
                updateMessage(String.format(Locale.ENGLISH, "Set mtu failed, mtu=%d, status=%d", mtu, status), false);
            }

            onGattServiceCharacteristicDiscovered();
        }
    }

    //设备数据回调
    private class WTROnlineCallbackMain extends WTROnlineCallback {
        @Override
        public void onGattPrepared(
                WTROnlineClient client,
                BluetoothGatt gatt
        ) {
            updateMessage("Discover service and characteristics success", false);
            int mtu = WTConstants.DEFAULT_MTU_LENGTH;
            mLog.d("Request MTU " + mtu);
            //必须设置mtu,默认值不足以发送最长数据,可根据实际应用调整
            boolean requestMtu = gatt.requestMtu(mtu);
            if (!requestMtu) {
                mLog.w("Request mtu failed");
                updateMessage(String.format(Locale.ENGLISH, "Request mtu %d failed", mtu), false);
                onGattServiceCharacteristicDiscovered();
            }
            super.onGattPrepared(client, gatt);
        }

        @Override
        public void onError(WTROnlineClient client, int errCode) {
            updateMessage(String.format(Locale.ENGLISH, "Receive error code %d", errCode), false);
            if (errCode == CODE_GATT_WRITE_TIMEOUT) {
                updateMessage("Gatt write timeout", false);
                client.close();
                onGattDisconnected();
            }
        }

        @Override
        public void onReceiveConfigNetworkResult(com.waytronic.libwtronline.WTROnlineClient client, boolean success) {
            super.onReceiveConfigNetworkResult(client, success);
            final String message = success ? "配置发送成功,请等待指示灯变化" : "配置失败";
            // 通知 WiFiConfigDialogFragment 配置结果
            ((WiFiConfigDialogFragment)getSupportFragmentManager().findFragmentByTag("WiFiConfigDialog"))
                    .notifyConfigResult(success, message);
            updateMessage(message, false);
            disconnectGatt();
        }
    }

    public static String bytesToHex(byte[] bytes) {
        StringBuilder hexString = new StringBuilder();
        for (byte b : bytes) {
            String hex = Integer.toHexString(0xff & b);
            if (hex.length() == 1) {
                hexString.append('0');
            }
            hexString.append(hex);
        }
        return hexString.toString();
    }


    private class WTButtonListener implements View.OnClickListener, View.OnLongClickListener {
        private Toast mToast;
        private ActivityResultLauncher<Intent> configActivityResultLauncher;
        @Override
        public void onClick(View v) {
            if (v == mContent.blufiConnect) {
                connect();
            } else if (v == mContent.blufiDisconnect) {
                disconnectGatt();
            } else if (v == mContent.blufiStartWificonfig) {
                WiFiConfigDialogFragment dialog = new WiFiConfigDialogFragment();
                dialog.show(getSupportFragmentManager(), "WiFiConfigDialog");
            }
        }

        @Override
        public boolean onLongClick(View v) {
            if (mToast != null) {
                mToast.cancel();
            }

            int msgRes = 0;
            if (v == mContent.blufiConnect) {
                msgRes = R.string.blufi_function_connect_msg;
            } else if (v == mContent.blufiDisconnect) {
                msgRes = R.string.blufi_function_disconnect_msg;
            } else if (v == mContent.blufiStartWificonfig) {
                msgRes = R.string.blufi_function_start_record;
            } else {
                return false;
            }

            mToast = Toast.makeText(WTActivity.this, msgRes, Toast.LENGTH_SHORT);
            mToast.show();

            return true;
        }
    }

    private static class MsgHolder extends RecyclerView.ViewHolder {
        TextView text1;

        MsgHolder(WtMessageItemBinding binding) {
            super(binding.getRoot());

            text1 = binding.text1;
        }
    }

    private class MsgAdapter extends RecyclerView.Adapter<MsgHolder> {

        @NonNull
        @Override
        public MsgHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            WtMessageItemBinding binding = WtMessageItemBinding.inflate(
                    getLayoutInflater(),
                    parent,
                    false
            );
            return new MsgHolder(binding);
        }

        @Override
        public void onBindViewHolder(@NonNull MsgHolder holder, int position) {
            Message msg = mMsgList.get(position);
            holder.text1.setText(msg.text);
            holder.text1.setTextColor(msg.isNotification ? Color.RED : Color.BLACK);
        }

        @Override
        public int getItemCount() {
            return mMsgList.size();
        }
    }

    private static class Message {
        String text;
        boolean isNotification;
    }
}
