package com.waytronic.wtronline_demo.ui;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.DialogFragment;

import android.os.Handler;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.waytronic.wtdemo.R;
public class WiFiConfigDialogFragment extends DialogFragment {

    private EditText wifiNameEditText;
    private EditText wifiPwdEditText;
    private ProgressBar progressBar;
    Button configButton;
    private WiFiConfigListener listener;

    private Handler handler = new Handler();
    private Runnable timeoutRunnable;
    private static final long TIMEOUT_MS = 1000; // 超时时间为4秒
    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (context instanceof WiFiConfigListener) {
            listener = (WiFiConfigListener) context;
        } else {
            throw new RuntimeException(context.toString() + " must implement WiFiConfigListener");
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        AlertDialog.Builder builder = new AlertDialog.Builder(requireActivity());

        // Inflate and set the layout for the dialog
        View view = LayoutInflater.from(getContext()).inflate(R.layout.dialog_wifi_config, null);
        builder.setView(view);

        wifiNameEditText = view.findViewById(R.id.wifiNameEditText);
        wifiPwdEditText = view.findViewById(R.id.wifiPwdEditText);
        progressBar = view.findViewById(R.id.progressBar);
        progressBar.setVisibility(View.INVISIBLE);

        configButton = view.findViewById(R.id.configButton);
        configButton.setOnClickListener(v -> handleConfigButtonClick(view));

        // Create and return the dialog
        AlertDialog dialog = builder.create();
        dialog.setCanceledOnTouchOutside(false);  // 点击空白处不关闭对话框
        dialog.setCancelable(false);             // 禁止按返回键关闭对话框（可选）
        return dialog;
    }

    private void handleConfigButtonClick(View rootView) {
        String wifiName = wifiNameEditText.getText().toString();
        String wifiPassword = wifiPwdEditText.getText().toString();

        if (wifiName.isEmpty() || wifiPassword.isEmpty()) {
            showToastMessage("请输入有效的WiFi名称和密码", Gravity.CENTER);
            return;
        }
        hideKeyboard(rootView); // 隐藏软键盘
        // 显示进度条
        progressBar.setVisibility(View.VISIBLE);
        this.getDialog().setCancelable(false);// 禁止按返回键关闭对话框（可选）
        this.getDialog().setCanceledOnTouchOutside(false);// 点击空白处不关闭对话框
        configButton.setEnabled(false);
        // 模拟异步任务来配置WiFi
        new Thread(() -> {
            try {
                startTimeoutTimer();
                // 调用 MainActivity 的配置方法
                if (listener != null) {
                    listener.startWiFiConfiguration(wifiName, wifiPassword);
                }
            } catch (Exception e) {
                configButton.setEnabled(true);
                e.printStackTrace();
                getActivity().runOnUiThread(() -> {
                    progressBar.setVisibility(View.INVISIBLE);
                    showToastMessage("配置失败", Gravity.CENTER);
                });
            }
        }).start();
    }
    private void startTimeoutTimer() {
        timeoutRunnable = () -> {
            if (getActivity() != null && isAdded()) {
                getActivity().runOnUiThread(() -> {
                    progressBar.setVisibility(View.INVISIBLE);
                    configButton.setEnabled(true);
                    showToastMessage("配置超时，请重试", Gravity.CENTER);
                    dismiss(); // 关闭对话框
                });
            }
        };
        handler.postDelayed(timeoutRunnable, TIMEOUT_MS);
    }

    private void hideKeyboard(View rootView) {
        if (rootView == null) return;

        InputMethodManager imm = (InputMethodManager) getContext().getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(rootView.getWindowToken(), 0);

    }
    // 提供给 MainActivity 回调配置结果的方法
    public void notifyConfigResult(boolean success, String message) {
        getActivity().runOnUiThread(() -> {
            progressBar.setVisibility(View.INVISIBLE);
            configButton.setEnabled(true);
            showToastMessage(message, Gravity.CENTER);
            if (success) {
                dismiss(); // 关闭对话框
            }
        });
    }
    private void showToastMessage(String message, int gravity) {
        Context context = getContext();
        if (context == null) return;

        Toast toast = Toast.makeText(context, message, Toast.LENGTH_SHORT);
        toast.setGravity(gravity, 0, 0); // 设置消息居中显示
        toast.show();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        handler.removeCallbacks(timeoutRunnable); // 确保在视图销毁时移除所有回调
    }
}

interface WiFiConfigListener {
    void startWiFiConfiguration(String wifiName, String wifiPassword);
}